%_______________________________________________________________________
%
% Configuration for running dotsExperiment
% Note that this is the configuration script for a single attribute during
% the training/psychophysics phase of the experiment.
% JQK July 2017 (kosciessa@mpib-berlin.mpg.de)
%_______________________________________________________________________
%
% Output
%
% dotInfo | configuration for random dots experiment (struct)
%
% Input
%
% PM - palmenides structure for adaptive psychophysics
%_______________________________________________________________________
%
% 170706    - added fixation color to blue
%           - added fields required by presentation

function dotInfo = createDotInfo_TEST_170704(PM, dim)

    dotInfo.trialsPerAtt = PM.numTrials;                                    % amount of trials is fixed by Palmenides
    dotInfo.numOfAtt = 1;                                                   % only one attribute during this test
    dotInfo.totalTrials = dotInfo.trialsPerAtt;
    dotInfo.curAtt = dim;
    dotInfo.dirAttentionRate = .5;                                          % actually not the case here, as no cue is presented, hence: 0
    
    dotInfo.durCue = 0;         % duration of cue
    dotInfo.durPres = .5;       % duration of presentation
    dotInfo.durResp = 2.5;      % duration of question
    dotInfo.durConf = 0;        % duration of confidence
    
    dotInfo.confOptions = '';       % 2 or 4 confidence options
    dotInfo.highlightChoice = 0;    % highlight choice by retaining only chosen option?
    
    dotInfo.dirSet          = [180, 0];     % dots in left or right direction
    dotInfo.numDotField     = 1;            % show a single dot patches on screen
    dotInfo.apXYD           = [0 0 130];    % coordinates and diameter of aperture
    dotInfo.speed           = [50];         % speed of dot motion
    dotInfo.trialtype       = [2 1 1];      % reaction time, not relevant, keyboard
    dotInfo.dotSize         = 3;            % dot size in pixels
    dotInfo.maxDotTime      = 2;            % maximum duration of moving dots
    dotInfo.fixXY           = [0 0];        % fixation coordinates
    dotInfo.fixDiam         = 2;            % fixation diameter
    dotInfo.fixColor        = [0 150 200];  % blue fixation dot
    dotInfo.fixMinTime      = 0.75;         % minimum fixation
    dotInfo.fixMaxTime      = 1.25;         % maximum fixation
    dotInfo.maxDotsPerFrame = 300;          % depends on graphics card
    dotInfo.fixTime         = 2;            % JQK: fixed onset fixation time

    % update frequency of on-screen content
    
    %dotInfo.Hz_BG   = 10; % background dots
    dotInfo.Hz_RDM  = 60; % kinematogram
    
    % multi-attribute task
    
    if dim == 1
        dotInfo.MAT.color = [255 255 255; 255 0 0];                         % define dot color (only modulated feature here)
    else
        dotInfo.MAT.color = [255 255 255; 255 255 255];
    end
    if dim == 2
        dotInfo.MAT.coherence = 1;
    else
        dotInfo.MAT.coherence = 0;
    end
    if dim == 3
        dotInfo.MAT.size = [5 8]; % define dot size
    else
        dotInfo.MAT.size = [8 8]; 
    end
    if dim == 4
        dotInfo.MAT.luminance = [.3 .8]; % define dot luminance
    else
        dotInfo.MAT.luminance = [.8 .8];
    end
    dotInfo.MAT.direction = [180 0];                                        % left and right
    dotInfo.MAT.attNames = {'color'; 'direction'; 'size'; 'luminance'};
    dotInfo.MAT.attNamesDE = {'Farbe'; 'Richtung'; 'Gre'; 'Helligkeit'};
    
    dotInfo.MAT.percAtt1H = .65;
    dotInfo.MAT.percAtt1L = 1-dotInfo.MAT.percAtt1H;
    dotInfo.MAT.percAtt2H = .65;
    dotInfo.MAT.percAtt2L = 1-dotInfo.MAT.percAtt2H;
    dotInfo.MAT.percAtt3H = .65;
    dotInfo.MAT.percAtt3L = 1-dotInfo.MAT.percAtt3H;
    dotInfo.MAT.percAtt4H = .65;
    dotInfo.MAT.percAtt4L = 1-dotInfo.MAT.percAtt4H;
    
    %% Keyboard responses
    % Use OS X keyboard naming scheme across all plattforms
    % Otherwise LeftControl/RightControl are not recognized
    KbName('UnifyKeyNames');

    if ismac % Mac keyboard
        dotInfo.keyLeft = [KbName('LeftControl'), KbName('LeftAlt'), KbName('LeftArrow')];      % left response
        dotInfo.keyRight = [KbName('RightControl'), KbName('RightAlt'), KbName('RightArrow')];  % right response
        dotInfo.keyConf1 = [KbName('LeftControl'), KbName('LeftArrow')];       % lowest confidence
        dotInfo.keyConf2 = KbName('LeftAlt');                                  % intermediate confidence low
        dotInfo.keyConf3 = KbName('RightAlt');                                 % intermediate confidence high
        dotInfo.keyConf4 = [KbName('RightControl'), KbName('RightArrow')];     % highest confidence
    else % PC keyboard
%         dotInfo.keyLeft = KbName('LeftControl');
%         dotInfo.keyRight = KbName('RightControl');
        dotInfo.keyLeft = [KbName('LeftControl'), KbName('LeftAlt'), KbName('LeftArrow')];
        dotInfo.keyRight = [KbName('RightControl'), KbName('RightAlt'), KbName('RightArrow')];
        dotInfo.keyConf1 = [KbName('LeftControl'), KbName('LeftArrow')];
        dotInfo.keyConf2 = KbName('LeftAlt');
        dotInfo.keyConf3 = KbName('RightAlt');
        dotInfo.keyConf4 = [KbName('RightControl'), KbName('RightArrow')];
    end

    dotInfo.keyModifier = KbName('LeftAlt'); % to prevent accidental input
    dotInfo.keyEscape = KbName('Escape'); %
    dotInfo.keyReturn = KbName('Return'); % continue experiment
    dotInfo.keyPause = KbName('p');

    %% randomize stimuli and non-target duration
    
    % set & retain random seed

    rseed = sum(100*clock);
    rng(rseed,'twister');
    [dotInfo.rngSetting] = rng;

    dotInfo.targetAtt = repmat(dotInfo.curAtt,1,dotInfo.trialsPerAtt);
    
    %% randomize directed attention trials & higher probability choice (within attribute)
    
    dotInfo.DirAttn = NaN(size(dotInfo.targetAtt));
    dotInfo.HighProbAtt = NaN(size(dotInfo.targetAtt));
    numSelAttn = ceil(dotInfo.dirAttentionRate*dotInfo.trialsPerAtt);
    for indAtt = dim
        tmp_trials = find(dotInfo.targetAtt == indAtt);
        if ~isempty(tmp_trials)
            tmp_rand = randperm(numel(tmp_trials));
            tmp_choice1 = tmp_trials(tmp_rand(1:numSelAttn));
            tmp_choice2 = tmp_trials(tmp_rand(numSelAttn+1:end));
            dotInfo.DirAttn(tmp_choice1) = 1;
            dotInfo.DirAttn(tmp_choice2) = 2;
            dotInfo.HighProbAtt(tmp_choice1(1:numel(tmp_choice1)/2)) = 1; % First half is designated highProb = 1;
            dotInfo.HighProbAtt(tmp_choice1((numel(tmp_choice1)/2)+1:end)) = 2; % Second half is deignated highProb = 2;
            dotInfo.HighProbAtt(tmp_choice2(1:numel(tmp_choice2)/2)) = 1;
            dotInfo.HighProbAtt(tmp_choice2((numel(tmp_choice1)/2)+1:end)) = 2;
        end
    end
        
    % Note that within attribute high probability choice and directed attention are orthogonal.
    % They both depend on the same randomization step.
    
    % The high probability choices only refer to the target attribute. The
    % high probability options for the other attributes are randomly drawn
    % below.
    
    % Note that there is no check that the conjunction of different
    % attributes appears equally often. 
    
    %% get random choices on each trial
    
    % This is not meaningful here, as each of the conjunction features are
    % set to an equal value.
    
    dotInfo.HighProbChoice = NaN(4,size(dotInfo.targetAtt,2));
    
    for indTrial = 1:dotInfo.totalTrials
        for indAtt = dotInfo.curAtt
            if dotInfo.targetAtt(indTrial) == indAtt
                dotInfo.HighProbChoice(indAtt,indTrial) = dotInfo.HighProbAtt(1,indTrial);
            else 
                tmp_rand = randperm(2);
                dotInfo.HighProbChoice(indAtt,indTrial) = tmp_rand(1);
            end
        end
    end
    
end
